<?php

#  Copyright (C) 2007 Jason Woofenden
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#  
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.


# This file is useful for putting message box(es) on the screen. If you include
# session_messages.php these messages can even persist accross http redirects
# (such as you should do after receiving a POST.
#
# Just call message("message here") whenever you have something to report.
#
#
# Once a template is loaded, call display_messages(). run.php will call
# display_messages() for you automaticallly after your main function returns
# (so long as you require_once this file.) If you want these messages to
# persist accross browser redirects, then you should
# require_once("code/wfpl/session_messages.php")
#
# Just make sure your template has a <!--~$messages {~--> section with a
# ~message html~ tag in it.
#
# Simple example:
#
#    <!--~$messages {~-->
#       <p>~message html~</p>
#    <!--~}~-->
#
# Full-featured example:
#
#    <!--~$messages once_if {~-->
#       <div id="wfpl_messages">
#          <!--~$messages {~-->
#             <p>~message html~</p>
#             <!--~ sep {~-->
#                <hr>
#             <!--~}~-->
#          <!--~}~-->
#       </div>
#    <!--~}~-->

require_once(__DIR__ . '/template.php');

# call this to display a message
function message($msg) {
	if(!isset($GLOBALS['wfpl_messages'])) {
		$GLOBALS['wfpl_messages'] = array();
	}

	$GLOBALS['wfpl_messages'][] = $msg;
}

# destructive
function get_messages() {
	if(!isset($GLOBALS['wfpl_messages'])) {
		$messages = array();
	} else {
		$messages = $GLOBALS['wfpl_messages'];
		unset($GLOBALS['wfpl_messages']);
	}

	if(function_exists('session_restore_messages')) {
		$messages = array_merge(session_restore_messages(), $messages);
	}

	return $messages;
}

# called automatically by run.php
function display_messages(&$tem = NULL, $key = '$messages') {
	if(!$tem) {
		$tem = &$GLOBALS['wfpl_template'];
	}
	$tem->data[$key] = columnize(get_messages(), 'message');
}
