<?php

#  Copyright (C) 2010 Jason Woofenden
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#  
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.


# This file contains tests for functions in code/wfpl/format.php
#
# See code/wfpl/test.php for details on how to run or create tests


require_once('code/wfpl/format.php');

function _test_format_money($arg, $dollars_correct, $money_correct) {
	unit_test_func('format_dollars', $arg, $dollars_correct);
	unit_test_func('format_money', $arg, $money_correct);
}

function test_format_money() {
	_test_format_money('1', '$1', '$1.00');
	_test_format_money('0', '$0', '$0.00');
	_test_format_money('0.0', '$0', '$0.00');
	_test_format_money('0.00', '$0', '$0.00');
	_test_format_money('00.00', '$0', '$0.00');
	_test_format_money('0.000', '$0', '$0.00');
	_test_format_money('0000.00000', '$0', '$0.00');
	_test_format_money('0.01', '$0', '$0.01');
	_test_format_money('0.51', '$1', '$0.51');
	_test_format_money('2.05', '$2', '$2.05');
	_test_format_money('2.10000500', '$2', '$2.10');
	_test_format_money('2.005', '$2', '$2.01');
	_test_format_money('2.004', '$2', '$2.00');
	_test_format_money('2.1', '$2', '$2.10');
	_test_format_money('2.0', '$2', '$2.00');
	_test_format_money('1000', '$1,000', '$1,000.00');
	_test_format_money('10000000', '$10,000,000', '$10,000,000.00');
	_test_format_money('100000000000', '$100,000,000,000', '$100,000,000,000.00');
	_test_format_money('1.99', '$2', '$1.99');
	_test_format_money('1.555', '$2', '$1.56');
	_test_format_money('19.555', '$20', '$19.56');
	_test_format_money('9.499', '$9', '$9.50');
	_test_format_money('9.989', '$10', '$9.99');
	_test_format_money('9.999', '$10', '$10.00');
	_test_format_money('19999999999.555', '$20,000,000,000', '$19,999,999,999.56');
}


# run tests where int, int_0, decimal, positive_decimal should return the same result
function _test_format_int($arg, $correct) {
	if($correct === '') {
		$correct_0 = '0';
	} else {
		$correct_0 = $correct;
	}
	unit_test_func('format_int', $arg, $correct);
	unit_test_func('format_int_0', $arg, $correct_0);
	unit_test_func('format_positive_decimal', $arg, $correct);
	unit_test_func('format_decimal', $arg, $correct);
}

function test_format_misc() {
	unit_test_func('format_caption', 'something', 'Something');
	unit_test_func('format_caption', 'two_words', 'Two Words');
	unit_test_func('format_caption', 'email_special_case', 'E-mail Special Case');
	unit_test_func('format_caption', 'BIG', 'BIG');
	unit_test_func('format_caption', '1,2.3%^&', '1,2.3%^&');

	# test int, int_0, positive_decimal, decimal
	_test_format_int('', '');
	_test_format_int('abcdef', '');
	_test_format_int('0', '0');
	_test_format_int('00', '0');
	_test_format_int('0000', '0');
	_test_format_int('1', '1');
	_test_format_int('100000000000000000000000000', '100000000000000000000000000');
	_test_format_int('10000 0000000000000000000000', '100000000000000000000000000');
	_test_format_int(' 1a2b3c%^&', '123');
	
	unit_test_func('format_int', '-1', '1');
	unit_test_func('format_int', '+1', '1');
	unit_test_func('format_int_0', '-1', '1');
	unit_test_func('format_int_0', '+1', '1');
	unit_test_func('format_positive_decimal', '-1', '1');
	unit_test_func('format_positive_decimal', '+1', '1');
	unit_test_func('format_decimal', '-1', '-1');
	unit_test_func('format_decimal', '+1', '1');

	unit_test_func('format_int', '-1.0', '1');
	unit_test_func('format_int', '+1.0', '1');
	unit_test_func('format_int_0', '-1.0', '1');
	unit_test_func('format_int_0', '+1.0', '1');
	unit_test_func('format_positive_decimal', '-1.0', '1.0');
	unit_test_func('format_positive_decimal', '+1.0', '1.0');
	unit_test_func('format_decimal', '-1.0', '-1.0');
	unit_test_func('format_decimal', '+1.0', '1.0');

	unit_test_func('format_positive_decimal', '1.0', '1.0');
	unit_test_func('format_decimal', '1.0', '1.0');

	# extra zeros after the decimal point
	unit_test_func('format_positive_decimal', '-1.00', '1.0');
	unit_test_func('format_positive_decimal', '+1.00', '1.0');
	unit_test_func('format_decimal', '-1.00', '-1.0');
	unit_test_func('format_decimal', '+1.00', '1.0');
	unit_test_func('format_positive_decimal', '-1.000', '1.0');
	unit_test_func('format_positive_decimal', '+1.000', '1.0');
	unit_test_func('format_decimal', '-1.000', '-1.0');
	unit_test_func('format_decimal', '+1.000', '1.0');
	# important zeros after the decimal point
	unit_test_func('format_positive_decimal', '-1.020', '1.02');
	unit_test_func('format_positive_decimal', '+1.020', '1.02');
	unit_test_func('format_decimal', '-1.020', '-1.02');
	unit_test_func('format_decimal', '+1.020', '1.02');
	unit_test_func('format_positive_decimal', '-1.0020', '1.002');
	unit_test_func('format_positive_decimal', '+1.0020', '1.002');
	unit_test_func('format_decimal', '-1.0020', '-1.002');
	unit_test_func('format_decimal', '+1.0020', '1.002');
	# signs in weird places
	unit_test_func('format_positive_decimal', '-1.-0020', '1.002');
	unit_test_func('format_positive_decimal', '+1.-0020', '1.002');
	unit_test_func('format_decimal', '-1.-0020', '-1.002');
	unit_test_func('format_decimal', '+1.-0020', '1.002');
	unit_test_func('format_positive_decimal', '-1.0-020', '1.002');
	unit_test_func('format_positive_decimal', '+1.0-020', '1.002');
	unit_test_func('format_decimal', '-1.0-020', '-1.002');
	unit_test_func('format_decimal', '+1.0-020', '1.002');
	unit_test_func('format_positive_decimal', '-1-.0020', '1.002');
	unit_test_func('format_positive_decimal', '+1-.0020', '1.002');
	unit_test_func('format_decimal', '-1-.0020', '-1.002');
	unit_test_func('format_decimal', '+1-.0020', '1.002');
	unit_test_func('format_positive_decimal', '--1.0020', '1.002');
	unit_test_func('format_positive_decimal', '+-1.0020', '1.002');
	unit_test_func('format_decimal', '--1.0020', '-1.002');
	unit_test_func('format_decimal', '+-1.0020', '-1.002');


	unit_test_func('format_zip', '', '');
	unit_test_func('format_zip', '12345', '12345');
	unit_test_func('format_zip', 'snt3nvwm4vm2zvw0%^&3-', '34203');
	unit_test_func('format_zip', '123456', '12345-6');
	unit_test_func('format_zip', '1234567890', '12345-67890');

	unit_test_func('format_filename', '1234567890', '1234567890');
	unit_test_func('format_filename', 'A*b#C:D/E?f<g&H!I', 'a_b_c_d_e_f_g_h_i');
	unit_test_func('format_filename', '-?', '__');
	unit_test_func('format_filename', '--help', '_-help');
	unit_test_func('format_filename', '.secret', '_secret');
	# allow uppercase:
	unit_test_func('format_filename', 'A*b#C:D/E?f<g&H!I', true, 'A_b_C_D_E_f_g_H_I');

	/* FIXME test these
	format_path($str, $allow_uppercase = false)
	client_path_to_filename($path)
	format_image_w_h($str)
	format_image_w_h_thumb_w_h($str)
	format_varname($str)
	format_oneline($str)
	format_unix($str)
	format_bool($str)
	format_yesno($str)
	format_email($str)
	format_url($str)
	strplusone($str)
	format_money($str, $display_cents = true)
	format_dollars($str)
	format_mdy_to_ymd($str)
	format_ymd($str)
	format_hours_minutes($str)
	format_hours($str)
	format_12hr_to_hours($str)
	format_phone($str)
	*/
}

function format_unit_tests_main() {
	test_format_money();
	test_format_misc();
}
